/*******************************************************************************
* File Name: ledPin.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "ledPin.h"

static ledPin_BACKUP_STRUCT  ledPin_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: ledPin_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet ledPin_SUT.c usage_ledPin_Sleep_Wakeup
*******************************************************************************/
void ledPin_Sleep(void)
{
    #if defined(ledPin__PC)
        ledPin_backup.pcState = ledPin_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            ledPin_backup.usbState = ledPin_CR1_REG;
            ledPin_USB_POWER_REG |= ledPin_USBIO_ENTER_SLEEP;
            ledPin_CR1_REG &= ledPin_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(ledPin__SIO)
        ledPin_backup.sioState = ledPin_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        ledPin_SIO_REG &= (uint32)(~ledPin_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: ledPin_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to ledPin_Sleep() for an example usage.
*******************************************************************************/
void ledPin_Wakeup(void)
{
    #if defined(ledPin__PC)
        ledPin_PC = ledPin_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            ledPin_USB_POWER_REG &= ledPin_USBIO_EXIT_SLEEP_PH1;
            ledPin_CR1_REG = ledPin_backup.usbState;
            ledPin_USB_POWER_REG &= ledPin_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(ledPin__SIO)
        ledPin_SIO_REG = ledPin_backup.sioState;
    #endif
}


/* [] END OF FILE */
